<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kronometer" version="7">

    <MenuBar>
        <Menu name="file">
            <text>&amp;File</text>
            <Action name="export_laps" />
        </Menu>
        <Menu name="kronometerActions" >
            <text>&amp;Kronometer</text>
            <Action name="start" />
            <Action name="pause" />
            <Action name="reset" />
            <Action name="lap" />
        </Menu>
    </MenuBar>

    <ToolBar position="top" iconSize="32" name="mainToolBar" noMerge="1">
        <text>Kronometer Toolbar</text>
        <Action name="start" />
        <Action name="pause" />
        <Action name="reset" />
        <Action name="lap" />
    </ToolBar>

    <!-- State rules definition (enabling/disabling actions) -->

    <State name="inactive" >
        <enable>
            <Action name="start" />
        </enable>

        <disable>
            <Action name="pause" />
            <Action name="reset" />
            <Action name="lap" />
            <Action name="file_save" />
            <Action name="file_save_as" />
            <Action name="export_laps" />
        </disable>
    </State>

    <State name="running" >
        <enable>
            <Action name="pause" />
            <Action name="reset" />
            <Action name="lap" />
        </enable>

        <disable>
            <Action name="start" />
            <Action name="file_save" />
            <Action name="file_save_as" />
            <Action name="export_laps" />
        </disable>
    </State>

    <State name="paused" >
        <enable>
            <Action name="start" />
            <Action name="reset" />
            <Action name="file_save_as" />
        </enable>

        <disable>
            <Action name="pause" />
            <Action name="lap" />
        </disable>
    </State>

    <!-- There is a session that has been modified and then paused -->
    <State name="pausedSession" >
        <enable>
            <Action name="start" />
            <Action name="reset" />
            <Action name="file_save" />
            <Action name="file_save_as" />
        </enable>

        <disable>
            <Action name="pause" />
            <Action name="lap" />
        </disable>
    </State>

</gui>
